// Copyright (c) 2021 10X Genomics, Inc. All rights reserved.

// Build html files by inserting other html files.
//
// If supplied the single argument BUILD, also rebuild from source.

use enclone_core::parse_bsv;
use enclone_core::testlist::SITE_EXAMPLES;
use enclone_tools::html::{edit_html, insert_html};
use io_utils::{fwrite, open_for_read, open_for_write_new};
use itertools::Itertools;
use pretty_trace::PrettyTrace;
use rayon::prelude::*;
use std::env;
use std::fs::{read_dir, File};
use std::io::{BufRead, Write};
use std::process::Command;
use string_utils::{strme, TextUtils};
use vector_utils::{bin_member, unique_sort};

fn main() {
    PrettyTrace::new().on();

    // Build from source.

    let args: Vec<String> = env::args().collect();
    if args.len() == 2 && args[1] == "BUILD" {
        let mut results = Vec::<(usize, Vec<u8>)>::new();
        for i in 0..SITE_EXAMPLES.len() {
            results.push((i, Vec::new()));
        }
        results.par_iter_mut().for_each(|r| {
            let i = r.0;
            let test = SITE_EXAMPLES[i].1;
            let args = parse_bsv(test);
            let new = Command::new("target/debug/enclone")
                .args(&args)
                .arg("MAX_CORES=24")
                .output()
                .unwrap_or_else(|_| panic!("{}", "failed to execute enclone".to_string()));
            if new.status.code() != Some(0) {
                eprint!(
                    "\nenclone_site_examples: example {} failed to execute, stderr =\n{}",
                    i + 1,
                    strme(&new.stderr),
                );
                std::process::exit(1);
            }
            r.1 = new.stdout;
        });
        for i in 0..SITE_EXAMPLES.len() {
            // Move file to the site location.

            let example_name = SITE_EXAMPLES[i].0;
            let out_file = example_name.to_string();
            let mut file = File::create(&out_file).unwrap();
            file.write_all(&results[i].1).unwrap();
        }
    }
    let mut site_ex = Vec::<String>::new();
    for i in 0..SITE_EXAMPLES.len() {
        let example_name = SITE_EXAMPLES[i].0;
        let out_file = example_name.to_string();
        site_ex.push(out_file);
    }
    unique_sort(&mut site_ex);

    // Apply insert_html.

    let all = read_dir("pages").unwrap();
    for f in all {
        let f = f.unwrap().path();
        let f = f.to_str().unwrap();
        if f.ends_with(".html.src") {
            let mut level = 2;
            let mut target = format!("pages/auto/{}.html", f.between("/", "."));
            if target == *"pages/auto/index.html" {
                target = "index.html".to_string();
                level = 0;
            }
            insert_html(f, &target, false, level);
        }
    }

    // This is ugly.  Edit the html pages.

    let all = read_dir("pages/auto").unwrap();
    let mut allx = vec!["index.html".to_string()];
    for f in all {
        let f = f.unwrap().path();
        let f = f.to_str().unwrap().to_string();
        if f.ends_with(".html") && !bin_member(&site_ex, &f) {
            allx.push(f);
        }
    }
    for f in allx {
        let mut edited = false;
        let mut lines = Vec::<String>::new();
        {
            let h = open_for_read![&f.to_string()];
            for line in h.lines() {
                let s = line.unwrap();
                lines.push(s.clone());
                if s.contains("googletag") {
                    edited = true;
                }
            }
        }
        if !edited {
            let x = format!("{}\n", lines.iter().format("\n"));
            let mut h = open_for_write_new![&f.to_string()];
            fwrite!(h, "{}", edit_html(&x));
        }
    }
}
